   --------------------------------------------------------------------
--|  Program : Moon Lander                     Version : 1.3c          
   --------------------------------------------------------------------
--|  Abstract :  A moon landing simulation.   The lander is being      
--|  pulled to the surface.  You must blast the engines to slow down   
--|  and try to land safely.  Refer to specification document for 
--|  details.
   --------------------------------------------------------------------
--|  File            : MOONLAND.ADB                                    
--|  Compiler/System : Meridian Ada / MS-DOS                                   
--|  Author          : John Dalbey             Date : 4/27/95          
   --------------------------------------------------------------------
--|  Limitations     :                                                 
--|      Due to conversion of float to integer in Display_Status       
--|      values for altitude, fuel, and velocity must stay within      
--|      the range for INTEGER or a constraint error will be raised.   
   --------------------------------------------------------------------
--|  Note: When using Meridian Ada, bind with the -u flag to           
--|        simulate having a math coprocessor, e.g.                    
--|        BAMP -u MOONLAND
--|                                                                    
--|  Version History  : V1.0 Original version
--|                   : V1.1 Design changes to analyze_landing
--|                   : V1.2 omit global variables           
--|                   : V1.3a,b,c 6/9/95 improved commenting and layout
--|                                                                    
   --------------------------------------------------------------------       
WITH Text_IO;   
USE  Text_IO;

PROCEDURE MoonLand IS

--
--    Constants used in this program are declared below.
--    Note: all units are metric (kgs, meters, etc.)
--
Lander_weight   : constant FLOAT := 5000.0;   -- weight of the spaceship
Fuel_conversion : constant FLOAT := 1000.0;   -- conversion factor
Gravity         : constant FLOAT :=    2.4;   -- acceleration due to gravity
Max_altitude    : constant FLOAT := 20000.0; -- limit of moon's gravity field

--
--    Make the numeric input/output packages available for this program
--
PACKAGE My_Int_IO IS NEW Text_io.Integer_IO ( Num => Integer);
PACKAGE My_Flt_IO IS NEW Text_io.Float_IO   ( Num => Float  );

-----------------------------------------------------------------------------
PROCEDURE Display_Instructions IS
--
-- Purpose: Display the instructions for how to play the game.
--          Display the column headers.
-- Assumes: nothing.
-- Returns: nothing.
--
BEGIN
   Set_Col(30);
   Put_Line ("Moon Lander Simulation");
   Put_Line ("Enter a positive integer for the amount of fuel to burn");
   Put_Line ("Try to land without crashing. ");
   New_Line;

   PUT (" TIME  FUEL  ALTITUDE   VELOCITY  DELTA_V  DELTA_ALT  BURN");
   New_Line;
END Display_Instructions;


-----------------------------------------------------------------------------
PROCEDURE Initialize
   ( Fuel     : OUT FLOAT;       -- fuel remaining (kilograms)
     Altitude : OUT FLOAT;       -- distance above the surface (meters)
     Velocity : OUT FLOAT;       -- speed and direction (meters/sec)
     Delta_alt: OUT FLOAT;       -- change in altitude
     Delta_v  : OUT FLOAT;       -- change in velocity
     Time     : OUT INTEGER) IS  -- elapsed time (seconds)
--
--
-- Purpose:   "Initialize" is used to reset the values for the six main
--            variables used in this program.  
-- Assumes: nothing.
-- Returns: Each variable is given an initial value. 
--
BEGIN
   Fuel      :=  5000.0;         -- kilograms
   Altitude  := 10000.0;         -- meters
   Velocity  :=     0.0;         -- meters/sec
   Delta_alt :=     0.0;         -- meters
   Delta_v   :=     0.0;         -- meters/sec
   Time      :=     0;           -- seconds
END Initialize;



-----------------------------------------------------------------------------
FUNCTION Landed
   (Altitude: IN FLOAT)  -- distance above surface
   RETURN BOOLEAN IS
--
-- Purpose:   "Landed" is used to check if the lander has reached the
--            surface.
-- Assumes:  Altitude contains the current distance above the surface. 
--           A positive value means the lander is still in the air. 
--           A negative value means it has landed.
-- Returns:  TRUE if the lander has reached the surface, 
--           and FALSE otherwise.
--
BEGIN
   RETURN Altitude <= 0.0 ;
END Landed;



-----------------------------------------------------------------------------
FUNCTION Failed
   (Altitude: IN FLOAT) -- distance above surface
   RETURN BOOLEAN IS
--
-- Purpose:   "Failed" is used to check if the mission has been aborted.
--            This is done by checking to see if the altitude of the
--            lander is too high, thus escaping the gravitational field
--            of the moon.
-- Assumes:  Altitude contains the current distance above the surface. 
--           A positive value means the lander is still in the air. 
--           A negative value means it has landed.
-- Returns:  Will return TRUE if the mission has been aborted,
--           (Altitude becomes too high, escapes moon's gravitational field)
--           and FALSE if otherwise.
--
--
BEGIN
   RETURN  Altitude > Max_altitude ;
END Failed;



-----------------------------------------------------------------------------
PROCEDURE Display_Status
   ( Fuel     : IN  FLOAT;       -- fuel remaining (kilograms)
     Altitude : IN  FLOAT;       -- distance above the surface (meters)
     Velocity : IN  FLOAT;       -- speed and direction (meters/sec)
     Delta_alt: IN  FLOAT;       -- change in altitude
     Delta_v  : IN  FLOAT;       -- change in velocity
     Time     : IN  INTEGER) IS  -- elapsed time (seconds)
--
-- Purpose:   Display the lander's current status, i.e., the current values 
--            of the six variables used in this program.  
-- Assumes:   The variables have been initialized. 
-- Returns:   Nothing.
--
BEGIN
   My_Int_IO.Put (Time, WIDTH=>5);
   Text_IO.Put   ("   ");
   My_Int_IO.Put (INTEGER(Fuel));
   Text_IO.Put   ("   ");
   My_Int_IO.Put (INTEGER(Altitude));
   Text_IO.Put   ("   ");
   My_Int_IO.Put (INTEGER(Velocity));
   Text_IO.Put   ("   ");
   My_Flt_IO.Put (Delta_v,  FORE=>3,AFT=>0,EXP=>0);
   Text_IO.Put   ("   ");
   My_Flt_IO.Put (Delta_alt,FORE=>3,AFT=>0,EXP=>0);
   Text_IO.Put   ("   ");
END Display_Status;


-----------------------------------------------------------------------------
PROCEDURE Compute_New_Position
   ( Fuel     : IN OUT FLOAT;       -- fuel remaining (kilograms)
     Altitude : IN OUT FLOAT;       -- distance above the surface (meters)
     Velocity : IN OUT FLOAT;       -- speed and direction (meters/sec)
     Delta_alt: IN OUT FLOAT;       -- change in altitude
     Delta_v  : IN OUT FLOAT;       -- change in velocity
     Time     : IN OUT INTEGER;     -- elapsed time (seconds)
     burn     : IN OUT INTEGER) IS  -- amount of fuel to burn
--
-- Purpose: Do the computations to get our new position, speed, etc.
-- Assumes: The variables have been initialized.
--          Burn contains the user's input.
-- Returns: The newly computed values for each of the variables,
--            based on the amount of fuel to burn.
--
BEGIN
   Fuel      := Fuel - ( FLOAT(Burn) * 10.0);

   Delta_v   := (FLOAT(burn) * 10.0 * Fuel_conversion) / (Lander_weight + Fuel)
                 -  (Gravity * 10.0);

   Delta_alt := (Velocity + Delta_v/2.0) * 10.0;

   Velocity  := Velocity + Delta_v;
   Altitude  := Altitude + Delta_alt;

   Time      := Time + 10;
END Compute_New_Position;



-----------------------------------------------------------------------------
PROCEDURE Validate_Burn
   (Burn: IN OUT INTEGER) IS  -- amount of fuel to burn
--
-- Purpose:   "Validate_Burn" is used to ensure that the user has entered
--            a valid amount of fuel to burn, that is, a value greater
--            than zero.  A warning message is displayed for invalid burn.
-- Assumes:   Burn contains the user's input for amount of fuel to burn.
-- Returns:   The amount to burn is set to zero if it is found to be
--            negative, and left alone if otherwise.  
--
BEGIN
   IF Burn < 0 THEN
      Put ("No cheating, now.  Your input for burn must be positive or zero.");
      Burn := 0;
   END IF;
   New_Line;
END Validate_Burn;




-----------------------------------------------------------------------------
PROCEDURE Analyze_Landing
   (Velocity: IN FLOAT;     -- speed and direction
    Altitude: IN FLOAT) IS  -- distance above the surface
--
-- Purpose:   "Analyze_Landing" displays a final message to the user based
--            on the speed and altitude of the lander.
-- Assumes:   The values for the velocity and altitude must be given.
-- Returns:   Nothing.
--
-- Algorithm:
--  IF Landed THEN
--       Depending on how fast the ship was traveling on impact,
--       display an appropriate message.
--  ELSE
--       display aborting (too high) message
--
BEGIN
   IF Landed (Altitude) THEN  -- Have we touched down?
      IF Velocity < -20.0 THEN  -- Were we travelling too fast?
         Put ("Crash landing -- Lander will never fly again.");
      ELSE
         Put ("You landed okay.");
      END IF;
   ELSE -- Didn't touchdown, so must have failed
      Put_Line("Too high -- You escaped the moon's gravity.");
      Put     ("            Have a nice trip around the galaxy!");
   END IF;
END Analyze_Landing;


--===========================================================================
--       Main Program Starts Here
-----------------------------------------------------------------------------
PROCEDURE Main IS
--
-- Purpose:   "Main" declares the variables used for the lander program
--            and calls all the appropriate functions and procedures to
--            run the program.  "Main" first displays instructions 
--            and then initializes six main variables.  These variables
--            represent fuel, altitude, speed and direction, change in
--            altitude, change in velocity, and elapsed time.
--
--            The interactive part of the program is then done by con-
--            tinuously showing the user the lander's status, asking the
--            user for an amount of fuel to burn, checking to see if the
--            amount is valid, and computing new values based on the
--            user's entry.
--
--            This goes on until the lander has either hit the surface
--            or sent out of moon's gravitational pull.  An analysis of
--            the landing is done in the end to check if the lander hit
--            the ground safely or not.
--
-- Assumes:   Nothing.
-- Returns:   Nothing.
--
--
--    Variable declarations
--
Fuel      : FLOAT;          -- fuel remaining (kilograms)
Altitude  : FLOAT;          -- distance above the surface (meters)
Velocity  : FLOAT;          -- speed and direction (meters/sec)
Delta_alt : FLOAT;          -- change in altitude
Delta_v   : FLOAT;          -- change in velocity
Time      : INTEGER;        -- elapsed time (seconds)
Burn      : INTEGER;        -- user's input: fuel to burn (kgs)

BEGIN  -- Main

   Display_Instructions;
   Initialize (Fuel, Altitude, Velocity, Delta_alt, Delta_v, Time);

   -- Repeat until the game is over
   WHILE NOT (Landed(Altitude) OR Failed(Altitude) ) LOOP

      Display_Status(Fuel, Altitude, Velocity, Delta_alt, Delta_v, Time);

      My_Int_IO.Get (Burn);       -- Obtain fuel to burn during next 10 seconds
      Validate_Burn (Burn);       -- Ensure it's a legal entry

      -- Do computations for new altitude, etc.
      Compute_New_Position(Fuel, Altitude, Velocity, Delta_alt, Delta_v, 
                           Time, Burn);

   END LOOP;

   -- When game ends, report on success/failure
   Analyze_Landing (Velocity, Altitude);

END Main;

BEGIN  -- Moon Lander
   Main;  -- Call Main program
END MoonLand;


